"""
    fuelwatcher - A python module for scraping XML data from the Western
    Australian governments Fuel Watch initiative.

    <https://www.fuelwatch.wa.gov.au>

        Copyright (C) 2018, Daniel Michaels
"""
from .constants import PRODUCT, REGION, BRAND, SUBURB
from xml.etree import ElementTree

import logging
import requests

logging.basicConfig(level=logging.INFO)


class FuelWatch:
    """Client for FuelWatch RSS Feed. """

    def __init__(self,
                 url='http://fuelwatch.wa.gov.au/fuelwatch/fuelWatchRSS',
                 product=PRODUCT, region=REGION, brand=BRAND, suburb=SUBURB):
        self.url = url
        self._product = product
        self._region = region
        self._brand = brand
        self._suburb = suburb

    def validate_product(self, product: int) -> bool:
        if not product:
            return True
        else:
            assert product in self._product, "Invalid Product Integer."

    def validate_region(self, region: int) -> bool:
        if not region:
            return True
        else:
            assert region in self._region, "Invalid Region Specified."

    def validate_brand(self, brand: int) -> bool:
        if not brand:
            return True
        else:
            assert brand in self._brand, "Invalid Brand."

    def validate_suburb(self, suburb: str) -> bool:
        if not suburb:
            return True
        else:
            assert suburb in self._suburb, "Invalid Suburb - Check Spelling"

    def query(self, product: int = None, suburb: str = None,
              region: int = None, brand: int = None, surrounding: str = None,
              day: str = None):
        """
        Returns FuelWatch data based on query parameters

        If all parameters are None it will return all stations with
        product set to Unleaded Petrol.

        :param product: Takes in a integer from the following table.
        1 - Unleaded Petrol     2 - Premium Unleaded
        4 - Diesel              5 - LPG
        6 - 98 RON              10 - E85
        11 - Brand diesel

        :param suburb: Takes a valid Western Australian suburb.

        Full list found in utils.suburbs

        :param region: FuelWatch seperates WA into regions that take an

        integer. Refer to utils.region for a listing.

        :param brand: Takes in any valid registered WA fuel station. Refer to
        utils.brand for the full list.

        :param surrounding: boolean 'yes/no' that will return surrounding
        suburbs when used in conjuction with the suburb parameter. Must be set
        to 'no' explicitly, otherwise returns True.

        :param day: Capable of four argument types:
            - today (this is the default)
            - tomorrow (only available after 2:30PM)
            - yesterday
            - DD/MM/YYYY (only prices for the last week, e.g. 23/08/2016)

            returns today if not set.

        :return: byte-string content of url
        """

        self.validate_product(product)
        self.validate_brand(brand)
        self.validate_region(region)
        self.validate_suburb(suburb)

        payload = dict()
        payload['Product'] = product
        payload['Suburb'] = suburb
        payload['Region'] = region
        payload['Brand'] = brand
        payload['Surrounding'] = surrounding
        payload['Day'] = day

        try:
            response = requests.get(self.url, timeout=30, params=payload)
            if response.status_code == 200:
                return response.content
        except Exception as e:
            print(e)

    def get_raw_xml(self, result: bytes):
        """
        Returns the full RSS response unparsed.

        :param result: url response.content from FuelWatch.query()

        :return: byte string full RSS XML response
        """
        return result

    def get_parsed_xml(self, result: bytes):
        """
        Given page content parses through the RSS XML and returns only 'item'
        data which contains fuel station information.

        :param result: url response.content from FuelWatch.query()

        :return: a list of dictionaries from the XML content.
        """

        dom = ElementTree.fromstring(result)
        items = dom.findall('channel/item')

        parsed_results = []
        for elem in items:
            dic = dict()

            dic['title'] = elem.find('title').text
            dic['description'] = elem.find('description').text
            dic['brand'] = elem.find('brand').text
            dic['date'] = elem.find('date').text
            dic['price'] = elem.find('price').text
            dic['trading-name'] = elem.find('trading-name').text
            dic['location'] = elem.find('location').text
            dic['address'] = elem.find('address').text
            dic['phone'] = elem.find('phone').text
            dic['latitude'] = elem.find('latitude').text
            dic['longitude'] = elem.find('longitude').text
            dic['site-features'] = elem.find('site-features').text
            parsed_results.append(dic)

        return parsed_results




PRODUCT = {
    1: "Unleaded Petrol",
    2: "Premium Unleaded",
    4: "Diesel",
    5: "LPG",
    6: "98 RON",
    7: "B20 Diesel"
}

REGION = {
    1: "Boulder",
    2: "Broome",
    3: "Busselton (Townsite)",
    4: "Carnarvon",
    5: "Collie",
    6: "Dampier",
    7: "Esperance",
    8: "Kalgoorlie",
    9: "Karratha",
    10: "Kununurra",
    11: "Narrogin",
    12: "Northam",
    13: "Port Hedland",
    14: "South Hedland",
    15: "Albany",
    16: "Bunbury",
    17: "Geraldton",
    18: "Mandurah",
    19: "Capel",
    20: "Dardanup",
    21: "Greenough",
    22: "Harvay",
    23: "Murray",
    24: "Waroona",
    25: "Metro : North of River",
    26: "Metro : South of River",
    27: "Metro : East/Hills",
    28: "Augusta / Margaret River",
    29: "Busselton (Shire)",
    30: "Bridgetown / Greenbushes",
    31: "Donnybrook / Balingup",
    32: "Manjimup",
    33: "Cataby",
    34: "Collgardie",
    35: "Cunderdin",
    36: "Dalwallinu",
    37: "Denmark",
    38: "Derby",
    39: "Dongara",
    40: "Exmouth",
    41: "Fitzroy Crossing",
    42: "Jurien",
    43: "Kambalda",
    44: "Kellerberrin",
    45: "Kojonup",
    46: "Meekatharra",
    47: "Moora",
    48: "Mt Barker",
    49: "Newman",
    50: "Norseman",
    51: "Ravensthorpe",
    53: "Tammin",
    54: "Williams",
    55: "Wubin",
    56: "York"
}

BRAND = {
    2: "Ampol",
    3: "Better Choice",
    4: "BOC",
    5: "BP",
    6: "Caltex",
    7: "Gull",
    8: "Kleenheat",
    9: "Kwikfuel",
    10: "Liberty",
    13: "Peak",
    14: "Shell",
    15: "Independent",
    16: "Wesco",
    19: "Caltex Woolworths",
    20: "Coles Express",
    21: "Black & White",
    23: "United",
    24: "Eagle"
}

SUBURB = [
    'Albany',
    'Alexander Heights',
    'Alfred Cove',
    'Applecross',
    'Armadale',
    'Ascot',
    'Attadale',
    'Augusta',
    'Australind',
    'Balcatta',
    'Baldivis',
    'Balga',
    'Balingup',
    'Ballajura',
    'Barragup',
    'Baskerville',
    'Bassendean',
    'Bayswater',
    'Beckenham',
    'Bedfordale',
    'Beechboro',
    'Beldon',
    'Bellevue',
    'Belmont',
    'Benger',
    'Bentley',
    'Bertram',
    'Bibra Lake',
    'Bicton',
    'Binningup',
    'Boulder',
    'Bouvard',
    'Boyanup',
    'Brentwood',
    'Bridgetown',
    'Broome',
    'Brunswick Junction',
    'Bull Creek',
    'Bullsbrook',
    'Bunbury',
    'Burswood',
    'Busselton',
    'Byford',
    'Canning Vale',
    'Cannington',
    'Capel',
    'Carbunup River',
    'Carine',
    'Carlisle',
    'Carnarvon',
    'Cataby',
    'Caversham',
    'Chidlow',
    'Claremont',
    'Clarkson',
    'Cloverdale',
    'Collie',
    'Como',
    'Coolgardie',
    'Coolup',
    'Cottesloe',
    'Cowaramup',
    'Cunderdin',
    'Currambine',
    'Dalwallinu',
    'Dampier',
    'Dardanup',
    'Dawesville',
    'Denmark',
    'Derby',
    'Dianella',
    'Dongara',
    'Donnybrook',
    'Doubleview',
    'Duncraig',
    'Dunsborough',
    'Dwellingup',
    'East Fremantle',
    'East Perth',
    'East Victoria Park',
    'Eaton',
    'Edgewater',
    'Ellenbrook',
    'Erskine',
    'Esperance',
    'Exmouth',
    'Falcon',
    'Fitzroy Crossing',
    'Floreat',
    'Forrestdale',
    'Forrestfield',
    'Fremantle',
    'Gelorup',
    'Geraldton',
    'Gidgegannup',
    'Girrawheen',
    'Glen Forrest',
    'Glendalough',
    'Glenfield',
    'Gnangara',
    'Golden Bay',
    'Gosnells',
    'Gracetown',
    'Greenbushes',
    'Greenough',
    'Greenwood',
    'Guildford',
    'Gwelup',
    'Halls Head',
    'Hamilton Hill',
    'Harvey',
    'Herne Hill',
    'High Wycombe',
    'Highgate',
    'Hillarys',
    'Huntingdale',
    'Innaloo',
    'Jandakot',
    'Jolimont',
    'Joondalup',
    'Jurien Bay',
    'Kalamunda',
    'Kalgoorlie',
    'Kambalda',
    'Karawara',
    'Kardinya',
    'Karragullen',
    'Karratha',
    'Karridale',
    'Karrinyup',
    'Kellerberrin',
    'Kelmscott',
    'Kewdale',
    'Kiara',
    'Kingsley',
    'Kirup',
    'Kojonup',
    'Koondoola',
    'Kununurra',
    'Kwinana',
    'Lakelands',
    'Langford',
    'Leda',
    'Leederville',
    'Leeming',
    'Lesmurdie',
    'Lynwood',
    'Maddington',
    'Madeley',
    'Malaga',
    'Mandurah',
    'Manjimup',
    'Manning',
    'Manypeaks',
    'Margaret River',
    'Meadow Springs',
    'Meekatharra',
    'Merriwa',
    'Middle Swan',
    'Midvale',
    'Mindarie',
    'Mirrabooka',
    'Moonyoonooka',
    'Moora',
    'Morley',
    'Mosman Park',
    'Mount Barker',
    'Mt Hawthorn',
    'Mt Helena',
    'Mt Lawley',
    'Mt Pleasant',
    'Mullaloo',
    'Mundaring',
    'Mundijong',
    'Munster',
    'Murdoch',
    'Myalup',
    'Myaree',
    'Narrogin',
    'Naval Base',
    'Nedlands',
    'Neerabup',
    'Newman',
    'Nollamara',
    'Noranda',
    'Norseman',
    'North Dandalup',
    'North Fremantle',
    'North Perth',
    'Northam',
    'Northbridge',
    'Northcliffe',
    'Nowergup',
    "O'Conner",
    'Ocean Reef',
    'Osborne Park',
    'Padbury',
    'Palmyra',
    'Parmelia',
    'Pearsall',
    'Pemberton',
    'Perth',
    'Picton',
    'Pinjarra',
    'Port Hedland',
    'Port Kennedy',
    'Preston Beach',
    'Quinns Rock',
    'Ravensthorpe',
    'Redcliffe',
    'Redmond',
    'Ridgewood',
    'Riverton',
    'Rivervale',
    'Rockingham',
    'Roleystone',
    'Rosa Brook',
    'Rottnest Island',
    'Sawyers Valley',
    'Scarborough',
    'Secret Harbour',
    'Serpentine',
    'Singleton',
    'Sorrento',
    'South Fremantle',
    'South Hedland',
    'South Lake',
    'South Perth',
    'South Yunderup',
    'Southern River',
    'Spearwood',
    'Stratham Downs',
    'Stratton',
    'Subiaco',
    'Success',
    'Swan View',
    'Swanbourne',
    'Tammin',
    'The Lakes',
    'Thornlie',
    'Tuart Hill',
    'Upper Swan',
    'Vasse',
    'Victoria Park',
    'Waikiki',
    'Walpole',
    'Wangara',
    'Wanneroo',
    'Warnbro',
    'Waroona',
    'Warwick',
    'Waterloo',
    'Wattle Grove',
    'Wedgefield',
    'Wellstead',
    'Welshpool',
    'Wembley',
    'West Perth',
    'West Swan',
    'Westfield',
    'Westminster',
    'Willetton',
    'Williams',
    'Witchcliffe',
    'Woodvale',
    'Wooroloo',
    'Wubin',
    'Yanchep',
    'Yokine',
    'York',
    'Young Siding',
    'Yunderup'
]



"""
Publish a new version:
$ git tag X.Y.Z -m "Release X.Y.Z"
$ git push --tags
$ pip install --upgrade twine wheel
$ python setup.py sdist bdist_wheel --universal
$ twine upload --repository-url https://test.pypi.org/legacy/ dist/*
    // ABOVE ONLY FOR TESTING
$ twine upload dist/*
"""

from codecs import open
from setuptools import setup
from os import path

__VERSION__ = '0.1.0rc3'
__URL__ = 'https://github.com/danielmichaels/fuelwatcher'
__DOWNLOAD_URL__ = (__URL__ + '/tarball/' + __VERSION__)

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='fuelwatcher',
    packages=['fuelwatcher'],
    version=__VERSION__,
    description='A simple XML scraper for FuelWatch.wa.gov.au fuel prices',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Daniel Michaels',
    author_email='dans.address@outlook.com',
    url=__URL__,
    download_url=__DOWNLOAD_URL__,
    keywords=['fuelwatch', 'western australia', 'fuel watch rss'],
    classifiers=[
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6', ]
)
